@@ -61,10 +61,16 @@ class MessageCode(Enum):
 BOARDS = [
     Boards.STM32F469I_DISCO.value,
     Boards.STM32H747I_DISCO.value,
+    Boards.STM32F407G_DISCO.value,
     Boards.IMXRT1050_EVKB.value,
 ]
 
-AUDIO_DEBUG_RECORD_DURATION_SEC = 3
+AUDIO_DEBUG_RECORD_DURATION_SEC = {
+    Boards.STM32F469I_DISCO.value: 3,
+    Boards.STM32H747I_DISCO.value: 3,
+    Boards.STM32F407G_DISCO.value: 1,
+    Boards.IMXRT1050_EVKB.value: 3
+}
 AUDIO_DEBUG_FFT_WINDOW_SIZE = 512
 AUDIO_DEBUG_FFT_OVERLAP = AUDIO_DEBUG_FFT_WINDOW_SIZE - 160
 AUDIO_DEBUG_SAMPLING_FREQUENCY = 16000
@@ -83,6 +89,7 @@ log.setLevel(logging.DEBUG if DEBUG else logging.WARNING)
 
 
 class PicoFrame(object):
+
     def consume(self, code, payload):
         raise NotImplementedError()
 
@@ -184,7 +191,7 @@ class PicovoiceApp(tk.Tk):
         self._menu_bar.add_cascade(label="Help", menu=self._help_menu, underline=0)
         self.config(menu=self._menu_bar)
 
-        self._status_bar = tk.Label(master=self, text='', bd=1, relief=tk.FLAT, anchor=tk.W)
+        self._status_bar = tk.Label(master=self, text='', bg=SOFT_GREY, bd=1, relief=tk.FLAT, anchor=tk.W)
         self._status_bar.grid(row=1, column=0, sticky='swe')
 
         self.protocol("WM_DELETE_WINDOW", self._destroy)
@@ -220,21 +227,21 @@ class PicovoiceApp(tk.Tk):
 
     def draw_upload_firmware_frame(self):
         old_frame = self._active_frame
-        self._active_frame = UploadFirmwareFrame(master=self)
+        self._active_frame = UploadFirmwareFrame(self)
         old_frame.destroy()
         del old_frame
 
-    def draw_upload_models_frame(self, board, message_thread):
+    def draw_upload_models_frame(self, board):
         self.update_status_bar("Connected to '%s'" % board)
         old_frame = self._active_frame
-        self._active_frame = UploadModelsFrame(master=self, board=board, message_thread=message_thread)
+        self._active_frame = UploadModelsFrame(self, board)
         old_frame.destroy()
         del old_frame
 
-    def draw_control_panel_frame(self, board, message_thread, picovoice_models):
+    def draw_control_panel_frame(self, board, picovoice_models):
         self.update_status_bar("Connected to '%s'" % board)
         old_frame = self._active_frame
-        self._active_frame = ControlPanel(self, board, message_thread, picovoice_models)
+        self._active_frame = ControlPanel(self, board, picovoice_models)
         old_frame.destroy()
         del old_frame
 
@@ -243,13 +250,7 @@ class UploadFirmwareFrame(tk.Frame, PicoFrame):
     def __init__(self, master):
         tk.Frame.__init__(self, master)
 
-        self._board = None
-        self._ports = None
-        self._port = None
-        self._serial_driver = None
-        self._message_thread = None
         self._is_uploading_done = False
-        self._is_connecting_done = False
 
         self.grid(row=0, column=0, sticky='nsew')
         self.grid_rowconfigure(0, weight=0)
@@ -263,14 +264,12 @@ class UploadFirmwareFrame(tk.Frame, PicoFrame):
         self.configure(**FRAME_COMMON_PARAMS)
 
         self._picovoice_image = ImageTk.PhotoImage(Image.open(IMAGES['picovoice']))
-
         self._picovoice_image_label = ttk.Label(
             self,
             image=self._picovoice_image,
             borderwidth=1,
             relief='flat',
             background=FRAME_COMMON_PARAMS['background'])
-
         self._picovoice_image_label.grid(row=0, column=0, columnspan=3, pady=20, sticky='s')
 
         self._hint_label = ttk.Label(
@@ -278,7 +277,6 @@ class UploadFirmwareFrame(tk.Frame, PicoFrame):
             text="Select Your Board",
             background=FRAME_COMMON_PARAMS['background'],
             font=(FONT[0], int(FONT[1] * 1.5)))
-
         self._hint_label.grid(row=1, column=0, columnspan=3, pady=20, sticky='s')
 
         self._boards_tree_view = ttk.Treeview(
@@ -308,15 +306,18 @@ class UploadFirmwareFrame(tk.Frame, PicoFrame):
     def _update_selected_board(self, _):
         selected_board = self._boards_tree_view.focus()
         self._board = self._boards_tree_view.item(selected_board)['text']
-        self._upload_button.state(["!disabled"])
-        self.master.update_status_bar("Selected '%s'" % self._board)
+        if self._board != '':
+            self._upload_button.state(["!disabled"])
+            self.master.update_status_bar("Selected '%s'" % self._board)
+        else:
+            self._board = None
 
     def _upload_firmware(self):
         self._uploading_label.start_loading()
-        self.master.update_status_bar('Uploading firmware ...')
+        self.master.update_status_bar('Uploading firmware')
         self._update_thread = threading.Thread(target=self._upload_helper)
         self._update_thread.start()
-        self._check_uploading()
+        self._check_uploading(1)
 
     def _upload_helper(self):
         self._upload_button.state(["disabled"])
@@ -336,86 +337,44 @@ class UploadFirmwareFrame(tk.Frame, PicoFrame):
         else:
             self._is_uploading_done = True
 
-    def _check_uploading(self):
+    def _check_uploading(self, n):
         if self._is_uploading_done:
-            sleep(0.1)
-            self.master.update_status_bar('Connecting to the board ...')
-            self._update_thread = threading.Thread(target=self._serial_port_select)
-            self._update_thread.start()
-            self._check_connecting()
-        else:
-            self.after(10, self._check_uploading)
-
-    @staticmethod
-    def _get_ports():
-        return serial.tools.list_ports.comports()
-
-    def _serial_port_select(self):
-        self._ports = self._get_ports()
-        for port in self._ports:
-            try:
-                self._serial_driver = serial.Serial(port=port.device, baudrate=115200)
-                self._message_thread = MessageControlThread(self._serial_driver, port, self)
-                self._message_thread.start_communicate()
-
-                for _ in range(SERIAL_CONNECTION_RETRIES):
-                    self._message_thread.send_message("%s 0000000000" % MessageCode.PV_MESSAGE_CODE_HANDSHAKE.value[0])
-                    sleep(0.2)
-                    if self._is_connecting_done:
-                        break
-
-                if self._is_connecting_done:
-                    break
-                else:
-                    self._message_thread.request_disconnect()
-
-            except serial.SerialException:
-                continue
-
-        if not self._is_connecting_done:
-            self._uploading_label.stop_loading()
-            messagebox.showerror(title='Error', message='No serial connection was detected for %s' % self._board)
-            self._boards_tree_view.state(["!disabled"])
-            self._upload_button.state(["!disabled"])
-            self._boards_tree_view.bind('<ButtonRelease-1>', self._update_selected_board)
-            self.master.update_status_bar("Selected '%s'" % self._board)
-
-    def _check_connecting(self):
-        if self._is_connecting_done:
-            self._uploading_label.stop_loading()
             self.master.update_status_bar("Connected to '%s'" % self._board)
             self.draw_next_frame()
         else:
-            self.after(10, self._check_connecting)
+            n = (n + 1) % 10
+            self.master.update_status_bar('Uploading firmware  ▏' + '▊' * n + '▁' * (9 - n) + '▕')
+            self.after(200, self._check_uploading, n)
 
     def clean(self):
-        if self._message_thread:
-            self._message_thread.request_disconnect()
+        pass
 
     def consume(self, code, payload):
-        if code is MessageCode.PV_MESSAGE_CODE_HANDSHAKE.value[0]:
-            if payload == self._board:
-                self._is_connecting_done = True
+        pass
 
     def connection_error(self):
         messagebox.showerror(title='Error', message='Connection to %s was lost' % self._board)
         self.master.draw_upload_firmware_frame()
 
     def draw_next_frame(self):
-        self._message_thread.change_consumer(None)
-        self.master.draw_upload_models_frame(self._board, self._message_thread)
+        self.clean()
+        self.master.draw_upload_models_frame(self._board)
 
     def draw_previous_frame(self):
         raise NotImplementedError()
 
 
 class UploadModelsFrame(tk.Frame, PicoFrame):
-    def __init__(self, master, board, message_thread):
+    def __init__(self, master, board):
         tk.Frame.__init__(self, master)
 
         self._board = board
-        self._message_thread = message_thread
-        self._uuid = None
+        self._ports = None
+        self._port = None
+        self._serial_driver = None
+        self._message_thread = None
+        self._is_connecting_done = False
+
         self._is_uploading_done = False
         self._is_responded = False
         self._selected_models = dict(porcupine=DEFAULT_PORCUPINE_MODEL, rhino=DEFAULT_RHINO_MODEL)
@@ -459,7 +418,8 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
             style="S.TButton",
             text='Copy',
             image=self._clipboard_photo,
-            compound='right'
+            compound='right',
+            width=10
         )
 
         self._uuid_copy_button.grid(row=0, column=2, padx=(0, 10), pady=20, ipady=4, sticky='sw')
@@ -502,10 +462,11 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
             command=self._select_porcupine_model,
             cursor='hand2',
             style="S.TButton",
-            text='Select...'
+            text=' Select... ',
+            width=12
         )
-
         self._select_porcupine_model_button.grid(row=1, column=1, padx=(0, 10), ipady=4)
+        self._select_porcupine_model_button.state(["disabled"])
 
         self._rhino_model_label = ttk.Label(
             self._upload_models_label_frame,
@@ -530,10 +491,11 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
             command=self._select_rhino_model,
             cursor='hand2',
             style="S.TButton",
-            text='Select...'
+            text=' Select... ',
+            width=12
         )
-
         self._select_rhino_model_button.grid(row=4, column=1, padx=(0, 10), pady=(0, 10), ipady=4,)
+        self._select_rhino_model_button.state(["disabled"])
 
         self._upload_label = LoadingAnimation(self)
         self._upload_label.grid(row=2, column=0, columnspan=3)
@@ -555,41 +517,77 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
             style="P.TButton",
             text='Use Default Models ⇨'
         )
-
         self._upload_models_button.grid(row=3, column=2, pady=20, padx=20, ipady=10, ipadx=10, sticky='se')
+        self._upload_models_button.state(["disabled"])
 
-        self._message_thread.change_consumer(self)
-        self.after(100, self._get_uuid)
+        self.master.update_status_bar('Connecting to the board')
+        self._update_thread = threading.Thread(target=self._connect)
+        self.after(2000, self._update_thread.start)
+        self._check_connecting(1)
+
+    def _connect(self):
+        for _ in range(SERIAL_CONNECTION_RETRIES):
+            for port in serial.tools.list_ports.comports():
+                try:
+                    self._port = port
+                    self._serial_driver = serial.Serial(port=port.device, baudrate=115200)
+                    self._message_thread = MessageControlThread(self._serial_driver, port, self)
+                    self._message_thread.start_communicate()
+                    self._message_thread.send_message("%s 0000000000"
+                                                      % MessageCode.PV_MESSAGE_CODE_HANDSHAKE.value[0])
+                    sleep(0.5)
+                except serial.SerialException:
+                    continue
+                if self._is_connecting_done:
+                    break
+                else:
+                    self._message_thread.request_disconnect()
+            if self._is_connecting_done:
+                break
+
+        if not self._is_connecting_done:
+            messagebox.showerror(title='Error', message='No serial connection was detected for %s' % self._board)
+            self.master.update_status_bar("Selected '%s'" % self._board)
+            self.master.draw_upload_firmware_frame()
+
+    def _check_connecting(self, n):
+        if self._is_connecting_done:
+            self.master.update_status_bar("Connected to '%s'" % self._board)
+            self.after(200, self._get_uuid)
+        else:
+            n = (n + 1) % 10
+            self.master.update_status_bar('Connecting to the board  ▏' + '▊' * n + '▁' * (9 - n) + '▕')
+            self.after(200, self._check_connecting, n)
 
     def _get_uuid(self):
         if not self._is_responded:
             self._message_thread.send_message("%s 0000000000" % MessageCode.PV_MESSAGE_CODE_UUID.value[0])
             self.after(200, self._get_uuid)
         else:
+            self._message_thread.change_consumer(None)
+            self._message_thread.request_disconnect()
             self._uuid_entry.state(["!disabled"])
             self._uuid_entry.delete(0, tk.END)
             self._uuid_entry.insert(0, self._uuid)
             self._uuid_entry.state(["disabled"])
             self._uuid_copy_button.state(["!disabled"])
+            self._upload_models_button.state(["!disabled"])
+            self._select_rhino_model_button.state(["!disabled"])
+            self._select_porcupine_model_button.state(["!disabled"])
 
-    def _check_uploading(self):
+    def _upload_models(self):
+        self.master.update_status_bar('Uploading models')
+        self.update_Thread = threading.Thread(target=self._upload_models_helper)
+        self.update_Thread.start()
+        self._check_uploading(1)
+
+    def _check_uploading(self, n):
         if self._is_uploading_done:
             self.draw_next_frame()
         else:
-            self.after(10, self._check_uploading)
-
-    def _copy_to_clipboard(self):
-        clip = Tk()
-        clip.withdraw()
-        clip.clipboard_clear()
-        clip.clipboard_append(self._uuid)
-        clip.destroy()
-
-    def _upload_models(self):
-        self.master.update_status_bar('Uploading models ...')
-        self.update_Thread = threading.Thread(target=self._upload_models_helper)
-        self.update_Thread.start()
-        self._check_uploading()
+            n = (n + 1) % 10
+            self.master.update_status_bar('Uploading models ▏' + '▊' * n + '▁' * (9 - n) + '▕')
+            self.after(200, self._check_uploading, n)
 
     def _upload_models_helper(self):
         if not NO_UPLOAD:
@@ -670,10 +668,21 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
             self._rhino_selected_model_entry.insert(0, '')
             self._upload_models_button.state(["!disabled"])
 
+    def _copy_to_clipboard(self):
+        clip = Tk()
+        clip.withdraw()
+        clip.clipboard_clear()
+        clip.clipboard_append(self._uuid)
+        clip.destroy()
+
     def consume(self, code, payload):
         if code is MessageCode.PV_MESSAGE_CODE_UUID.value[0]:
             self._is_responded = True
             self._uuid = payload
+            self.clean()
+        elif code is MessageCode.PV_MESSAGE_CODE_HANDSHAKE.value[0]:
+            if payload == self._board:
+                self._is_connecting_done = True
 
     def clean(self):
         if self._message_thread:
@@ -684,25 +693,28 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
         self.master.draw_upload_firmware_frame()
 
     def draw_next_frame(self):
-        self._message_thread.change_consumer(None)
-        self.master.draw_control_panel_frame(self._board, self._message_thread, self._selected_models)
+        self.clean()
+        self.master.draw_control_panel_frame(self._board, self._selected_models)
 
     def draw_previous_frame(self):
-        self._message_thread.change_consumer(None)
-        self._message_thread.request_disconnect()
+        self.clean()
         self.master.draw_upload_firmware_frame()
 
 
 class ControlPanel(tk.Frame, PicoFrame):
-    def __init__(self, master, board, message_thread, models):
+    def __init__(self, master, board, models):
         tk.Frame.__init__(self, master)
 
         self._board = board
+        self._ports = None
+        self._port = None
+        self._serial_driver = None
+        self._message_thread = None
+        self._is_connecting_done = False
+        self._is_responded = False
         self._models = models
-        self._message_thread = message_thread
         self._rhino_context = None
         self._context_viewer = None
-        self._is_responded = False
         self._monitoring_center_frame = MonitoringCenter(self, self._board, self._models)
         self._audio_debugging_frame = AudioDebugging(self, self._board)
 
@@ -738,15 +750,50 @@ class ControlPanel(tk.Frame, PicoFrame):
             style="S.TButton",
             text='Show Context...'
         )
-
         self._show_context_button.grid(row=1, column=1, pady=(5, 20), padx=20, ipady=10, ipadx=10, sticky='se')
-        self._message_thread.change_consumer(self)
-        self.after(200, self._get_context)
+        self._show_context_button.state(["disabled"])
+
+        self.master.update_status_bar('Connecting to the board')
+        self._update_thread = threading.Thread(target=self._connect)
+        self.after(2000, self._update_thread.start)
+        self._check_connecting(1)
+
+    def _connect(self):
+        for _ in range(SERIAL_CONNECTION_RETRIES):
+            for port in serial.tools.list_ports.comports():
+                try:
+                    self._port = port
+                    self._serial_driver = serial.Serial(port=port.device, baudrate=115200)
+                    self._message_thread = MessageControlThread(self._serial_driver, port, self)
+                    self._message_thread.start_communicate()
+                    self._message_thread.send_message("%s 0000000000"
+                                                      % MessageCode.PV_MESSAGE_CODE_HANDSHAKE.value[0])
+                    sleep(0.5)
+                except serial.SerialException:
+                    continue
+                if self._is_connecting_done:
+                    break
+                else:
+                    self._message_thread.request_disconnect()
+            if self._is_connecting_done:
+                break
+
+        if not self._is_connecting_done:
+            messagebox.showerror(title='Error', message='No serial connection was detected for %s' % self._board)
+            self.master.update_status_bar("Selected '%s'" % self._board)
+            self.master.draw_upload_firmware_frame()
+
+    def _check_connecting(self, n):
+        if self._is_connecting_done:
+            self.master.update_status_bar("Connected to '%s'" % self._board)
+            self.after(200, self._get_context)
+        else:
+            n = (n + 1) % 10
+            self.master.update_status_bar('Connecting to the board  ▏' + '▊' * n + '▁' * (9 - n) + '▕')
+            self.after(200, self._check_connecting, n)
 
     def _get_context(self):
         if not self._is_responded:
-            self._message_thread.send_message("%s 0000000000" % MessageCode.PV_MESSAGE_CODE_HANDSHAKE.value[0])
-            sleep(0.2)
             self._message_thread.send_message("%s 0000000000" % MessageCode.PV_MESSAGE_CODE_CONTEXT.value[0])
             self.after(200, self._get_context)
 
@@ -803,9 +850,13 @@ class ControlPanel(tk.Frame, PicoFrame):
     }
 
     def consume(self, code, payload):
-        if code is MessageCode.PV_MESSAGE_CODE_CONTEXT.value[0]:
+        if code is MessageCode.PV_MESSAGE_CODE_HANDSHAKE.value[0]:
+            if payload == self._board:
+                self._is_connecting_done = True
+        elif code is MessageCode.PV_MESSAGE_CODE_CONTEXT.value[0]:
             self._is_responded = True
             self._rhino_context = payload
+            self._show_context_button.state(["!disabled"])
         elif code in self._MONITORING_CENTER_CODES:
             self._monitoring_center_frame.consume(code, payload)
         elif code in [MessageCode.PV_MESSAGE_CODE_AUDIO_DUMP.value[0]]:
@@ -824,8 +875,8 @@ class ControlPanel(tk.Frame, PicoFrame):
 
     def draw_previous_frame(self):
         self._context_exit()
-        self._message_thread.change_consumer(None)
-        self.master.draw_upload_models_frame(self._board, self._message_thread)
+        self.clean()
+        self.master.draw_upload_models_frame(self._board)
 
 
 class MonitoringCenter(tk.Frame):
@@ -943,9 +994,11 @@ class MonitoringCenter(tk.Frame):
         if code == '[DEBUG]':
             self._log_scrolled_text.insert('end', ' %s\n' % payload, 'DEBUG')
             self._log_scrolled_text.see('end')
+
         elif code == '[WARN]':
             self._log_scrolled_text.insert('end', ' %s\n' % payload, 'WARN')
             self._log_scrolled_text.see('end')
+
         elif code == '[ERROR]':
             self._log_scrolled_text.insert('end', ' %s\n' % payload, 'ERROR')
             self._log_scrolled_text.see('end')
@@ -953,8 +1006,10 @@ class MonitoringCenter(tk.Frame):
             self._porcupine_sensitivity_scale.configure(state=tk.DISABLED, takefocus=0)
             self._rhino_sensitivity_scale.configure(state=tk.DISABLED, takefocus=0)
             self._clear_button.state(["disabled"])
+
         elif code is MessageCode.PV_MESSAGE_CODE_INFO.value[0]:
             pass
+
         elif code is MessageCode.PV_MESSAGE_CODE_WAKE_DETECTED.value[0]:
             self._log_scrolled_text.delete('1.0', 'end')
             self._log_scrolled_text.insert('end', ' \"%s\" detected ...\n' % self._models['porcupine'])
@@ -982,7 +1037,6 @@ class MonitoringCenter(tk.Frame):
                     self._log_scrolled_text.insert('end', '   }\n')
             self._log_scrolled_text.insert('end', ' }\n\n')
             self._log_scrolled_text.insert('end', ' %s \"%s\"... \n\n' % ('Listening for ', self._models['porcupine']))
-
             self._log_scrolled_text.see('end')
 
         elif code is MessageCode.PV_MESSAGE_CODE_PORCUPINE_SENSITIVITY.value[0]:
@@ -1000,7 +1054,7 @@ class MonitoringCenter(tk.Frame):
                     MONITORING_CENTER_SMOOTHING_FACTOR_CPU_USAGE * self._old_cpu
                 self._old_cpu = self._cpu_usage_progress_bar['value']
 
-        if code is MessageCode.PV_MESSAGE_CODE_VOLUME.value[0]:
+        elif code is MessageCode.PV_MESSAGE_CODE_VOLUME.value[0]:
             try:
                 volume_percentage = (
                         (int(payload) - MONITORING_CENTER_VU_METER_MIN_DB) * 100 / (-MONITORING_CENTER_VU_METER_MIN_DB))
@@ -1011,6 +1065,15 @@ class MonitoringCenter(tk.Frame):
             except ValueError as error:
                 log.debug(error)
 
+        elif code is MessageCode.PV_MESSAGE_CODE_ERROR.value[0]:
+            print('hi')
+            self._log_scrolled_text.insert('end', ' %s\n' % payload, 'ERROR')
+            self._log_scrolled_text.see('end')
+            self.master.invalid_state()
+            self._porcupine_sensitivity_scale.configure(state=tk.DISABLED, takefocus=0)
+            self._rhino_sensitivity_scale.configure(state=tk.DISABLED, takefocus=0)
+            self._clear_button.state(["disabled"])
+
 
 class AudioDebugging(tk.Frame):
     def __init__(self, master, board):
@@ -1113,11 +1176,12 @@ class AudioDebugging(tk.Frame):
         n = n - 1
         if self._is_downloading:
             if n < 0:
+
                 self.master.send_message("%s 0000000000" % MessageCode.PV_MESSAGE_CODE_AUDIO_DUMP.value[0])
                 self.master.master.update_status_bar('Recording ...')
                 self._hint_label_text.set('%s is now recording your voice ...' % self._board)
-                self.after(3000, self.update_status_bar_downloading, 0)
-                self.after(3000, self._downloading_label.start_loading)
+                self.after(AUDIO_DEBUG_RECORD_DURATION_SEC[self._board] * 1000, self.update_status_bar_downloading, 0)
+                self.after(AUDIO_DEBUG_RECORD_DURATION_SEC[self._board] * 1000, self._downloading_label.start_loading)
             else:
                 self.after(1000, self.update_status_bar_recording, n)
         else:
@@ -1132,7 +1196,7 @@ class AudioDebugging(tk.Frame):
         self._audio_downloading['value'] = n
         n = n + 1
         if self._is_downloading:
-            self.after(AUDIO_DEBUG_RECORD_DURATION_SEC * 70, self.update_status_bar_downloading, n)
+            self.after(AUDIO_DEBUG_RECORD_DURATION_SEC[self._board] * 70, self.update_status_bar_downloading, n)
         else:
             self._audio_downloading.grid_forget()
             self._downloading_label.stop_loading()
@@ -1321,6 +1385,9 @@ class MessageControlThread(threading.Thread):
         elif text.startswith(MessageCode.PV_MESSAGE_CODE_AUDIO_DUMP.value[0]):
             code = MessageCode.PV_MESSAGE_CODE_AUDIO_DUMP.value[0]
             message_context = text[5:]
+        elif text.startswith(MessageCode.PV_MESSAGE_CODE_ERROR.value[0]):
+            code = MessageCode.PV_MESSAGE_CODE_ERROR.value[0]
+            message_context = text[5:]
         else:
             code = None
             message_context = None
